/**
*	file: City.java
*
*	A class that represents statistical data for a city, population, area, and population
*	density
*
*	@author: D. Slater
*	@version: 3/31/2003
*/

import java.text.*;

public class City {

	// instance variables
	private String name;
	private int population;
	private double area;
	
	/**
	*	builds a city object with the supplied parameters
	*
	*	@param String cityName - the name of the city
	*	@param int cityPop - the population of the city
	*	@param double cityArea - the area in square miles of the city
	*/
	public City (String cityName, int cityPop, double cityArea) {
	
		name = cityName;
		population = cityPop;
		area = cityArea;
	}
	
	/**
	*	returns a String representation of the instance variables (state) of the city
	*
	*	@return String - the current values of this city object
	*/
	public String toString() {
	
		String str  = name + "   " + population + "  " + area + "   " + getPopDensity();
		return str;
	}
	
	// modifiers (setters - change the current values of the instance valraiables to 
	//                                  different values)
	
	public void setPopulation (int newPopulation) {
	
		population = newPopulation;
	}
	
	public void setArea(double newArea) {
	
		area = newArea;
	}
	
	// accessors (getters - return the current values of the instanve variables)
	
	public String getName() {
	
		return name;
	}
	
	public int getPopulation() {
	
		return population;
	}
	
	public double getArea() {
	
		return area;
	}
	
	public double getPopDensity() {
	
		return population / area;
	}
	
	
	// predicates (returns boolean value (true or false) related to this object)
	
	/**
	*	compares the name of this city with the other city and returns true
	*	if the names are true, returns false otherwise. 
	*	Ignores the case of the name for comparison
	*
	*	@param City other - the city to be compared against
	*	@return boolean - true if the city names are equal, regardless of case,
	*	false otherwise
	*/
	
	public boolean equals(City other) {
	
		return name.equalsIgnoreCase(other.getName());
	}
	
	/**
	*	compares the population desnity of this city with the other city and returns value
	*	less than 0 if this city's pop density is less than the others, value greater than 0
	*	if pop desnity of this city is greater than the other, and 0 if both are equal
	*	Ignores the case of the name for comparison
	*
	*	@param City other - the city to be compared against
	*	@return int - value less than 0 if this < other, greater than 0 if this > other,
	*	0 if they are equal
	*/
	
	public int compareTo(City other) {
	
		return (int)Math.round(getPopDensity() - other.getPopDensity());
	}
	
	
	// others (methods that provide useful behaviors for the object)
	
	/**
	*	adjusts the population of the city by the given percentage adjustment; 
	*	the adjustment can be either a positive or negative value
	*
	*	@param double pctAdjustment - the percentage amount the population is to change
	*/
	public void adjustPopulation(double pctAdjustment) {
	
	
		population = population + (int)(Math.round(population * pctAdjustment));
	}



}